package java.awt;

import java.awt.geom.Rectangle2D;
import java.text.CharacterIterator;
import java.awt.font.LineMetrics;
import java.awt.font.FontRenderContext;
import java.io.Serializable;

public abstract class FontMetrics implements Serializable
{
    private static final FontRenderContext DEFAULT_FRC;
    protected Font font;
    private static final long serialVersionUID = 1681126225205050147L;
    
    protected FontMetrics(Font font) {
        this.font = font;
    }
    
    public Font getFont() {
        return this.font;
    }
    
    public FontRenderContext getFontRenderContext() {
        return FontMetrics.DEFAULT_FRC;
    }
    
    public int getLeading() {
        return 0;
    }
    
    public int getAscent() {
        return this.font.getSize();
    }
    
    public int getDescent() {
        return 0;
    }
    
    public int getHeight() {
        return this.getLeading() + this.getAscent() + this.getDescent();
    }
    
    public int getMaxAscent() {
        return this.getAscent();
    }
    
    public int getMaxDescent() {
        return this.getDescent();
    }
    
    @Deprecated
    public int getMaxDecent() {
        return this.getMaxDescent();
    }
    
    public int getMaxAdvance() {
        return -1;
    }
    
    public int charWidth(int n) {
        if (!Character.isValidCodePoint(n)) {
            n = 65535;
        }
        if (n < 256) {
            return this.getWidths()[n];
        }
        char[] dst = new char[2];
        return this.charsWidth(dst, 0, Character.toChars(n, dst, 0));
    }
    
    public int charWidth(char c) {
        if (c < 'Ā') {
            return this.getWidths()[c];
        }
        return this.charsWidth(new char[] { c }, 0, 1);
    }
    
    public int stringWidth(String s) {
        int length = s.length();
        char[] dst = new char[length];
        s.getChars(0, length, dst, 0);
        return this.charsWidth(dst, 0, length);
    }
    
    public int charsWidth(char[] value, int offset, int count) {
        return this.stringWidth(new String(value, offset, count));
    }
    
    public int bytesWidth(byte[] ascii, int offset, int count) {
        return this.stringWidth(new String(ascii, 0, offset, count));
    }
    
    public int[] getWidths() {
        int[] array = new int[256];
        for (int i = '\0'; i < 'Ā'; i = (char)(i + 1)) {
            array[i] = this.charWidth((char)i);
        }
        return array;
    }
    
    public boolean hasUniformLineMetrics() {
        return this.font.hasUniformLineMetrics();
    }
    
    public LineMetrics getLineMetrics(String s, Graphics graphics) {
        return this.font.getLineMetrics(s, this.myFRC(graphics));
    }
    
    public LineMetrics getLineMetrics(String s, int n, int n2, Graphics graphics) {
        return this.font.getLineMetrics(s, n, n2, this.myFRC(graphics));
    }
    
    public LineMetrics getLineMetrics(char[] array, int n, int n2, Graphics graphics) {
        return this.font.getLineMetrics(array, n, n2, this.myFRC(graphics));
    }
    
    public LineMetrics getLineMetrics(CharacterIterator characterIterator, int n, int n2, Graphics graphics) {
        return this.font.getLineMetrics(characterIterator, n, n2, this.myFRC(graphics));
    }
    
    public Rectangle2D getStringBounds(String s, Graphics graphics) {
        return this.font.getStringBounds(s, this.myFRC(graphics));
    }
    
    public Rectangle2D getStringBounds(String s, int n, int n2, Graphics graphics) {
        return this.font.getStringBounds(s, n, n2, this.myFRC(graphics));
    }
    
    public Rectangle2D getStringBounds(char[] array, int n, int n2, Graphics graphics) {
        return this.font.getStringBounds(array, n, n2, this.myFRC(graphics));
    }
    
    public Rectangle2D getStringBounds(CharacterIterator characterIterator, int n, int n2, Graphics graphics) {
        return this.font.getStringBounds(characterIterator, n, n2, this.myFRC(graphics));
    }
    
    public Rectangle2D getMaxCharBounds(Graphics graphics) {
        return this.font.getMaxCharBounds(this.myFRC(graphics));
    }
    
    private FontRenderContext myFRC(Graphics graphics) {
        if ((graphics instanceof Graphics2D)) {
            return ((Graphics2D)graphics).getFontRenderContext();
        }
        return FontMetrics.DEFAULT_FRC;
    }
    
    @Override
    public String toString() {
        return this.getClass().getName() + "[font=" + this.getFont() + "ascent=" + this.getAscent() + ", descent=" + this.getDescent() + ", height=" + this.getHeight() + "]";
    }
    
    private static native void initIDs();
    
    static {
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        DEFAULT_FRC = new FontRenderContext(null, false, false);
    }
}
